from __future__ import annotations

from dataclasses import dataclass
from enum import IntEnum
from typing import Any


class ExitCode(IntEnum):
    SUCCESS = 0
    USAGE_ERROR = 2
    AUTH_ERROR = 3
    API_ERROR = 4
    TASK_FAILED = 5
    TIMEOUT = 6
    DOWNLOAD_ERROR = 7


@dataclass
class CliError(Exception):
    message: str
    exit_code: ExitCode = ExitCode.API_ERROR
    code: str = "api_error"
    details: Any = None

    def __str__(self) -> str:
        return self.message



from __future__ import annotations

import json
import os
from dataclasses import dataclass
from pathlib import Path

from simplicity_cli.errors import CliError, ExitCode
from simplicity_cli.models import GlobalOptions

DEFAULT_BASE_URL = "https://api.simplicity.ai"
API_KEY_ENV_VAR = "SIMPLICITY_AI_API_KEY"
XDG_CONFIG_HOME_ENV_VAR = "XDG_CONFIG_HOME"
CONFIG_DIR_NAME = "simplicity-cli"
CONFIG_FILE_NAME = "config.json"


@dataclass(frozen=True)
class RuntimeConfig:
    api_key: str
    base_url: str
    request_timeout_seconds: float
    json_output: bool


def resolve_runtime_config(options: GlobalOptions) -> RuntimeConfig:
    if options.request_timeout_seconds <= 0:
        raise CliError(
            "request timeout must be greater than 0 seconds.",
            exit_code=ExitCode.USAGE_ERROR,
            code="invalid_timeout",
        )

    api_key = (
        options.api_key
        or os.getenv(API_KEY_ENV_VAR, "")
        or load_saved_api_key()
        or ""
    ).strip()
    if not api_key:
        raise CliError(
            f"missing API key. Set --api-key, {API_KEY_ENV_VAR}, or run `simplicity-cli --api-key <key>` once.",
            exit_code=ExitCode.USAGE_ERROR,
            code="missing_api_key",
        )

    return RuntimeConfig(
        api_key=api_key,
        base_url=DEFAULT_BASE_URL,
        request_timeout_seconds=options.request_timeout_seconds,
        json_output=options.json_output,
    )


def load_saved_api_key() -> str | None:
    config_path = get_config_path()
    if not config_path.exists():
        return None

    try:
        payload = json.loads(config_path.read_text(encoding="utf-8"))
    except (OSError, json.JSONDecodeError):
        return None

    if not isinstance(payload, dict):
        return None
    api_key = payload.get("api_key")
    if isinstance(api_key, str):
        normalized = api_key.strip()
        return normalized or None
    return None


def save_api_key(api_key: str) -> Path:
    normalized = api_key.strip()
    if not normalized:
        raise CliError(
            "API key cannot be empty.",
            exit_code=ExitCode.USAGE_ERROR,
            code="invalid_api_key",
        )

    config_path = get_config_path()
    config_path.parent.mkdir(parents=True, exist_ok=True)
    payload = {"api_key": normalized}
    try:
        config_path.write_text(json.dumps(payload, ensure_ascii=True, indent=2) + "\n", encoding="utf-8")
        os.chmod(config_path, 0o600)
    except OSError as exc:
        raise CliError(
            f"failed to save API key to {config_path}.",
            exit_code=ExitCode.USAGE_ERROR,
            code="api_key_save_failed",
            details={"path": str(config_path), "error": str(exc)},
        ) from exc
    return config_path


def get_config_path() -> Path:
    config_home = os.getenv(XDG_CONFIG_HOME_ENV_VAR)
    if config_home:
        base = Path(config_home).expanduser()
    else:
        base = Path.home() / ".config"
    return base / CONFIG_DIR_NAME / CONFIG_FILE_NAME



from __future__ import annotations

import json
from pathlib import Path
from typing import Callable

import typer

from simplicity_cli.client import SimplicityApiClient
from simplicity_cli.config import resolve_runtime_config, save_api_key
from simplicity_cli.errors import CliError, ExitCode
from simplicity_cli.models import GlobalOptions
from simplicity_cli.output import OutputReporter
from simplicity_cli.workflows import (
    load_optional_text,
    run_fill_existing,
    run_fill_new,
    run_task_status,
    run_task_wait,
)

app = typer.Typer(
    name="simplicity-cli",
    help=(
        "CLI for AI-assisted PDF form filling.\n\n"
        "Auth: pass --api-key or set SIMPLICITY_AI_API_KEY.\n"
        "Pass --api-key once to save it for future runs.\n"
        "Primary commands: new, existing, status, wait.\n"
        "Run `simplicity-cli help` for a one-page guide."
    ),
    no_args_is_help=False,
    add_completion=False,
)
legacy_fill_app = typer.Typer(name="fill", no_args_is_help=True)
legacy_task_app = typer.Typer(name="task", no_args_is_help=True)
app.add_typer(legacy_fill_app, name="fill", hidden=True)
app.add_typer(legacy_task_app, name="task", hidden=True)

FULL_HELP_TEXT = """simplicity-cli one-page help

Auth:
  --api-key <KEY>  (or SIMPLICITY_AI_API_KEY env var)
  Run `simplicity-cli --api-key <KEY>` once to save login locally.

Primary commands:
  simplicity-cli new ...
  simplicity-cli existing FORM_ID ...
  simplicity-cli status TASK_ID
  simplicity-cli wait TASK_ID

Command: new
  simplicity-cli new (--form-file PATH | --form-url URL) [options]
  Rules:
    - Exactly one of --form-file/--form-url is required.
    - Provide at least one source via:
      --source-file/--source-url OR --context/--context-file
    - --context and --context-file are mutually exclusive.
    - --instructions and --instructions-file are mutually exclusive.
    - --output cannot be used with --no-download.
  Common examples:
    simplicity-cli new --form-file ./form.pdf --context "Applicant: Jane Doe"
    simplicity-cli new --form-file ./form.pdf --source-file ./w2.pdf --source-file ./id.pdf
    simplicity-cli new --form-url https://example.com/form.pdf --source-url https://example.com/source.pdf --no-wait --no-download

Command: existing
  simplicity-cli existing FORM_ID [options]
  Common examples:
    simplicity-cli existing FORM_ID --context "Use latest profile data"
    simplicity-cli existing FORM_ID --context-file ./context.txt --output ./filled.pdf
    simplicity-cli existing FORM_ID --no-wait --no-download

Command: status
  simplicity-cli status TASK_ID

Command: wait
  simplicity-cli wait TASK_ID [--poll-interval-seconds 2] [--max-wait-seconds 1800]

JSON output:
  Add --json to any command for one machine-readable JSON object.

Legacy aliases (still supported):
  simplicity-cli fill new ...
  simplicity-cli fill existing ...
  simplicity-cli task status ...
  simplicity-cli task wait ...
"""


@app.callback(invoke_without_command=True)
def root_callback(
    ctx: typer.Context,
    api_key: str | None = typer.Option(
        None,
        "--api-key",
        help="API key for X-API-Key auth. Overrides SIMPLICITY_AI_API_KEY.",
    ),
    json_output: bool = typer.Option(False, "--json", help="Emit strict JSON output."),
    request_timeout_seconds: float = typer.Option(
        60.0,
        "--request-timeout-seconds",
        min=0.001,
        help="Per-request timeout in seconds.",
    ),
) -> None:
    ctx.obj = GlobalOptions(
        api_key=api_key,
        json_output=json_output,
        request_timeout_seconds=request_timeout_seconds,
    )

    if api_key:
        try:
            config_path = save_api_key(api_key)
        except CliError as exc:
            raise _as_typer_exit(ctx, exc, command="login") from exc

        if ctx.invoked_subcommand is None:
            if json_output:
                typer.echo(
                    json.dumps(
                        {
                            "command": "login",
                            "ok": True,
                            "status": "saved",
                            "config_path": str(config_path),
                        },
                        ensure_ascii=True,
                    )
                )
            else:
                typer.echo(f"API key saved to {config_path}")
            raise typer.Exit(code=0)

    if ctx.invoked_subcommand is None:
        typer.echo(ctx.get_help())
        raise typer.Exit(code=0)


@app.command("help", help="Print a complete one-page command guide.")
def help_command() -> None:
    typer.echo(FULL_HELP_TEXT)


@app.command(
    "new",
    help=(
        "Create and autofill a new form from a PDF file or URL.\n\n"
        "Rules:\n"
        "- Provide exactly one of --form-file or --form-url.\n"
        "- Provide at least one source: --source-file/--source-url OR --context/--context-file.\n"
        "- Use either --context or --context-file (not both).\n"
        "- Use either --instructions or --instructions-file (not both).\n"
        "- --output cannot be combined with --no-download.\n\n"
        "Examples:\n"
        "  simplicity-cli new --form-file ./form.pdf --context \"Applicant: Jane Doe\"\n"
        "  simplicity-cli new --form-file ./form.pdf --source-file ./w2.pdf --source-file ./id.pdf\n"
        "  simplicity-cli new --form-url https://example.com/form.pdf --source-url https://example.com/source.pdf --no-wait --no-download"
    ),
)
def new_command(
    ctx: typer.Context,
    form_file: Path | None = typer.Option(
        None,
        "--form-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Local path to the PDF form.",
    ),
    form_url: str | None = typer.Option(None, "--form-url", help="URL to the form file."),
    source_file: list[Path] = typer.Option(
        [],
        "--source-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Source file path(s) used for autofill context.",
    ),
    source_url: list[str] = typer.Option(
        [],
        "--source-url",
        help="Source file URL(s) used for autofill context.",
    ),
    context: str | None = typer.Option(None, "--context", help="Free-text context for autofill."),
    context_file: Path | None = typer.Option(
        None,
        "--context-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Path to a text file with autofill context.",
    ),
    instructions: str | None = typer.Option(None, "--instructions", help="AI instructions override."),
    instructions_file: Path | None = typer.Option(
        None,
        "--instructions-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Path to a text file with AI instructions.",
    ),
    no_wait: bool = typer.Option(False, "--no-wait", help="Do not wait for autofill completion."),
    poll_interval_seconds: float = typer.Option(
        2.0,
        "--poll-interval-seconds",
        min=0.001,
        help="Polling interval while waiting for tasks.",
    ),
    max_wait_seconds: float = typer.Option(
        1800.0,
        "--max-wait-seconds",
        min=0.001,
        help="Maximum polling duration before timeout.",
    ),
    no_download: bool = typer.Option(False, "--no-download", help="Do not download completed form."),
    output: Path | None = typer.Option(
        None,
        "--output",
        help="Output file path for downloaded PDF.",
    ),
) -> None:
    context_text = load_optional_text(context, context_file, "context")
    instructions_text = load_optional_text(instructions, instructions_file, "instructions")

    if (form_file is None) == (form_url is None):
        raise _as_typer_exit(
            ctx,
            CliError(
                "provide exactly one of --form-file or --form-url.",
                exit_code=ExitCode.USAGE_ERROR,
                code="invalid_input",
            ),
            command="fill new",
        )
    if not source_file and not source_url and not context_text:
        raise _as_typer_exit(
            ctx,
            CliError(
                "fill new requires at least one source input (--source-file/--source-url) or --context.",
                exit_code=ExitCode.USAGE_ERROR,
                code="invalid_input",
            ),
            command="fill new",
        )
    if no_download and output is not None:
        raise _as_typer_exit(
            ctx,
            CliError(
                "cannot use --output with --no-download.",
                exit_code=ExitCode.USAGE_ERROR,
                code="invalid_input",
            ),
            command="fill new",
        )

    _execute(
        ctx=ctx,
        command="fill new",
        runner=lambda client, reporter: run_fill_new(
            client,
            reporter,
            form_file=form_file,
            form_url=form_url,
            source_files=list(source_file),
            source_urls=list(source_url),
            context=context_text,
            instructions=instructions_text,
            wait=not no_wait,
            poll_interval_seconds=poll_interval_seconds,
            max_wait_seconds=max_wait_seconds,
            download=not no_download,
            output_path=output,
        ),
    )


@app.command(
    "existing",
    help=(
        "Autofill an existing form by form ID.\n\n"
        "Rules:\n"
        "- FORM_ID is required.\n"
        "- Use either --context or --context-file (not both).\n"
        "- Use either --instructions or --instructions-file (not both).\n"
        "- --output cannot be combined with --no-download.\n\n"
        "Examples:\n"
        "  simplicity-cli existing FORM_ID --context \"Use latest profile data\"\n"
        "  simplicity-cli existing FORM_ID --context-file ./context.txt --output ./filled.pdf\n"
        "  simplicity-cli existing FORM_ID --no-wait --no-download"
    ),
)
def existing_command(
    ctx: typer.Context,
    form_id: str = typer.Argument(..., help="Existing form ID."),
    context: str | None = typer.Option(None, "--context", help="Free-text context for autofill."),
    context_file: Path | None = typer.Option(
        None,
        "--context-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Path to a text file with autofill context.",
    ),
    instructions: str | None = typer.Option(None, "--instructions", help="AI instructions override."),
    instructions_file: Path | None = typer.Option(
        None,
        "--instructions-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Path to a text file with AI instructions.",
    ),
    no_wait: bool = typer.Option(False, "--no-wait", help="Do not wait for autofill completion."),
    poll_interval_seconds: float = typer.Option(
        2.0,
        "--poll-interval-seconds",
        min=0.001,
        help="Polling interval while waiting for tasks.",
    ),
    max_wait_seconds: float = typer.Option(
        1800.0,
        "--max-wait-seconds",
        min=0.001,
        help="Maximum polling duration before timeout.",
    ),
    no_download: bool = typer.Option(False, "--no-download", help="Do not download completed form."),
    output: Path | None = typer.Option(
        None,
        "--output",
        help="Output file path for downloaded PDF.",
    ),
) -> None:
    context_text = load_optional_text(context, context_file, "context")
    instructions_text = load_optional_text(instructions, instructions_file, "instructions")
    if no_download and output is not None:
        raise _as_typer_exit(
            ctx,
            CliError(
                "cannot use --output with --no-download.",
                exit_code=ExitCode.USAGE_ERROR,
                code="invalid_input",
            ),
            command="fill existing",
        )

    _execute(
        ctx=ctx,
        command="fill existing",
        runner=lambda client, reporter: run_fill_existing(
            client,
            reporter,
            form_id=form_id,
            context=context_text,
            instructions=instructions_text,
            wait=not no_wait,
            poll_interval_seconds=poll_interval_seconds,
            max_wait_seconds=max_wait_seconds,
            download=not no_download,
            output_path=output,
        ),
    )


@app.command(
    "status",
    help=(
        "Fetch current status/details for a task ID.\n\n"
        "Example:\n"
        "  simplicity-cli status TASK_ID"
    ),
)
def status_command(ctx: typer.Context, task_id: str = typer.Argument(..., help="Task ID.")) -> None:
    _execute(
        ctx=ctx,
        command="task status",
        runner=lambda client, _reporter: run_task_status(client, task_id=task_id),
    )


@app.command(
    "wait",
    help=(
        "Poll a task until it reaches a terminal state.\n\n"
        "Example:\n"
        "  simplicity-cli wait TASK_ID --poll-interval-seconds 2 --max-wait-seconds 1800"
    ),
)
def wait_command(
    ctx: typer.Context,
    task_id: str = typer.Argument(..., help="Task ID."),
    poll_interval_seconds: float = typer.Option(
        2.0,
        "--poll-interval-seconds",
        min=0.001,
        help="Polling interval while waiting for tasks.",
    ),
    max_wait_seconds: float = typer.Option(
        1800.0,
        "--max-wait-seconds",
        min=0.001,
        help="Maximum polling duration before timeout.",
    ),
) -> None:
    _execute(
        ctx=ctx,
        command="task wait",
        runner=lambda client, reporter: run_task_wait(
            client,
            reporter,
            task_id=task_id,
            poll_interval_seconds=poll_interval_seconds,
            max_wait_seconds=max_wait_seconds,
        ),
    )


def _execute(
    *,
    ctx: typer.Context,
    command: str,
    runner: Callable[[SimplicityApiClient, OutputReporter], dict[str, object]],
) -> None:
    options = _options_from_context(ctx)
    reporter = OutputReporter(options.json_output)
    try:
        config = resolve_runtime_config(options)
        with SimplicityApiClient(config) as client:
            payload = runner(client, reporter)
        reporter.emit_success(command, payload)
    except CliError as exc:
        raise _as_typer_exit(ctx, exc, command=command) from exc


def _as_typer_exit(ctx: typer.Context, error: CliError, command: str) -> typer.Exit:
    options = _options_from_context(ctx)
    reporter = OutputReporter(options.json_output)
    reporter.emit_error(command, error)
    return typer.Exit(code=int(error.exit_code))


def _options_from_context(ctx: typer.Context) -> GlobalOptions:
    options = ctx.obj
    if isinstance(options, GlobalOptions):
        return options
    return GlobalOptions(api_key=None, json_output=False, request_timeout_seconds=60.0)


@legacy_fill_app.command("new", help="Legacy alias for `new`.")
def legacy_fill_new_command(
    ctx: typer.Context,
    form_file: Path | None = typer.Option(
        None,
        "--form-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Local path to the PDF form.",
    ),
    form_url: str | None = typer.Option(None, "--form-url", help="URL to the form file."),
    source_file: list[Path] = typer.Option(
        [],
        "--source-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Source file path(s) used for autofill context.",
    ),
    source_url: list[str] = typer.Option(
        [],
        "--source-url",
        help="Source file URL(s) used for autofill context.",
    ),
    context: str | None = typer.Option(None, "--context", help="Free-text context for autofill."),
    context_file: Path | None = typer.Option(
        None,
        "--context-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Path to a text file with autofill context.",
    ),
    instructions: str | None = typer.Option(None, "--instructions", help="AI instructions override."),
    instructions_file: Path | None = typer.Option(
        None,
        "--instructions-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Path to a text file with AI instructions.",
    ),
    no_wait: bool = typer.Option(False, "--no-wait", help="Do not wait for autofill completion."),
    poll_interval_seconds: float = typer.Option(
        2.0,
        "--poll-interval-seconds",
        min=0.001,
        help="Polling interval while waiting for tasks.",
    ),
    max_wait_seconds: float = typer.Option(
        1800.0,
        "--max-wait-seconds",
        min=0.001,
        help="Maximum polling duration before timeout.",
    ),
    no_download: bool = typer.Option(False, "--no-download", help="Do not download completed form."),
    output: Path | None = typer.Option(
        None,
        "--output",
        help="Output file path for downloaded PDF.",
    ),
) -> None:
    new_command(
        ctx=ctx,
        form_file=form_file,
        form_url=form_url,
        source_file=source_file,
        source_url=source_url,
        context=context,
        context_file=context_file,
        instructions=instructions,
        instructions_file=instructions_file,
        no_wait=no_wait,
        poll_interval_seconds=poll_interval_seconds,
        max_wait_seconds=max_wait_seconds,
        no_download=no_download,
        output=output,
    )


@legacy_fill_app.command("existing", help="Legacy alias for `existing`.")
def legacy_fill_existing_command(
    ctx: typer.Context,
    form_id: str = typer.Argument(..., help="Existing form ID."),
    context: str | None = typer.Option(None, "--context", help="Free-text context for autofill."),
    context_file: Path | None = typer.Option(
        None,
        "--context-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Path to a text file with autofill context.",
    ),
    instructions: str | None = typer.Option(None, "--instructions", help="AI instructions override."),
    instructions_file: Path | None = typer.Option(
        None,
        "--instructions-file",
        file_okay=True,
        dir_okay=False,
        exists=True,
        readable=True,
        resolve_path=True,
        help="Path to a text file with AI instructions.",
    ),
    no_wait: bool = typer.Option(False, "--no-wait", help="Do not wait for autofill completion."),
    poll_interval_seconds: float = typer.Option(
        2.0,
        "--poll-interval-seconds",
        min=0.001,
        help="Polling interval while waiting for tasks.",
    ),
    max_wait_seconds: float = typer.Option(
        1800.0,
        "--max-wait-seconds",
        min=0.001,
        help="Maximum polling duration before timeout.",
    ),
    no_download: bool = typer.Option(False, "--no-download", help="Do not download completed form."),
    output: Path | None = typer.Option(
        None,
        "--output",
        help="Output file path for downloaded PDF.",
    ),
) -> None:
    existing_command(
        ctx=ctx,
        form_id=form_id,
        context=context,
        context_file=context_file,
        instructions=instructions,
        instructions_file=instructions_file,
        no_wait=no_wait,
        poll_interval_seconds=poll_interval_seconds,
        max_wait_seconds=max_wait_seconds,
        no_download=no_download,
        output=output,
    )


@legacy_task_app.command("status", help="Legacy alias for `status`.")
def legacy_task_status_command(ctx: typer.Context, task_id: str = typer.Argument(..., help="Task ID.")) -> None:
    status_command(ctx=ctx, task_id=task_id)


@legacy_task_app.command("wait", help="Legacy alias for `wait`.")
def legacy_task_wait_command(
    ctx: typer.Context,
    task_id: str = typer.Argument(..., help="Task ID."),
    poll_interval_seconds: float = typer.Option(
        2.0,
        "--poll-interval-seconds",
        min=0.001,
        help="Polling interval while waiting for tasks.",
    ),
    max_wait_seconds: float = typer.Option(
        1800.0,
        "--max-wait-seconds",
        min=0.001,
        help="Maximum polling duration before timeout.",
    ),
) -> None:
    wait_command(
        ctx=ctx,
        task_id=task_id,
        poll_interval_seconds=poll_interval_seconds,
        max_wait_seconds=max_wait_seconds,
    )


def main() -> None:
    app()


if __name__ == "__main__":
    main()



from __future__ import annotations

import json
import sys
from typing import Any

import typer

from simplicity_cli.errors import CliError
from simplicity_cli.models import JSONDict

SUMMARY_FIELDS = ("task_id", "form_id", "document_id", "download_url", "download_path")
_SPINNER_FRAMES = "|/-\\"
_TERMINAL_STATUSES = {"completed", "failed"}


class OutputReporter:
    def __init__(self, json_output: bool) -> None:
        self._json_output = json_output
        self._phase_index = 0
        self._supports_live_updates = (not json_output) and sys.stdout.isatty()
        self._live_task_id: str | None = None
        self._live_line_len = 0
        self._spinner_index = 0

    def phase(self, message: str) -> None:
        if not self._json_output:
            self._flush_live_line()
            self._phase_index += 1
            typer.echo(f"[{self._phase_index}] {message}")

    def detail(self, message: str) -> None:
        if not self._json_output:
            self._flush_live_line()
            typer.echo(f"    {message}")

    def task_update(
        self,
        *,
        label: str,
        task_id: str,
        status: str,
        progress: int | None,
        elapsed_seconds: float,
    ) -> None:
        if self._json_output:
            return

        if not self._supports_live_updates:
            progress_display = ""
            if progress is not None:
                progress_display = f", {progress}%"
            typer.echo(
                f"    {label}: {status}{progress_display} (task_id={task_id}, elapsed={int(elapsed_seconds)}s)"
            )
            return

        if self._live_task_id is not None and self._live_task_id != task_id:
            self._flush_live_line()

        self._live_task_id = task_id
        line = (
            f"\r    {label}: {self._format_progress_indicator(progress)} "
            f"{status:<12} elapsed {int(elapsed_seconds)}s"
        )
        padding = max(0, self._live_line_len - len(line))
        typer.echo(f"{line}{' ' * padding}", nl=False)
        self._live_line_len = len(line)

        if status in _TERMINAL_STATUSES:
            self._flush_live_line()

    def emit_success(self, command: str, payload: JSONDict) -> None:
        result = self._normalize_payload(command=command, ok=True, payload=payload, error_payload=None)
        if self._json_output:
            typer.echo(json.dumps(result, ensure_ascii=True))
            return

        self._flush_live_line()
        typer.echo("")
        self._print_human_summary(result)

    def emit_error(self, command: str, error: CliError) -> None:
        details = error.details if isinstance(error.details, dict) else {}
        error_payload = {
            "code": error.code,
            "message": error.message,
            "details": error.details,
        }
        payload = {
            "status": "error",
            "task_id": details.get("task_id"),
            "task": details.get("task"),
            "form_id": details.get("form_id"),
            "document_id": details.get("document_id"),
            "download_url": details.get("download_url"),
            "download_path": details.get("download_path"),
        }
        result = self._normalize_payload(
            command=command,
            ok=False,
            payload=payload,
            error_payload=error_payload,
        )
        if self._json_output:
            typer.echo(json.dumps(result, ensure_ascii=True))
            return

        self._flush_live_line()
        typer.echo(f"error: {error.message}")
        if error.code:
            typer.echo(f"code: {error.code}")
        for field in SUMMARY_FIELDS:
            value = result.get(field)
            if value:
                typer.echo(f"{field}: {value}")

    def _normalize_payload(
        self,
        *,
        command: str,
        ok: bool,
        payload: JSONDict,
        error_payload: JSONDict | None,
    ) -> JSONDict:
        return {
            "command": command,
            "ok": ok,
            "status": payload.get("status"),
            "task_id": payload.get("task_id"),
            "task": payload.get("task"),
            "form_id": payload.get("form_id"),
            "document_id": payload.get("document_id"),
            "download_url": payload.get("download_url"),
            "download_path": payload.get("download_path"),
            "error": error_payload,
        }

    @staticmethod
    def _print_human_summary(result: JSONDict) -> None:
        status = result.get("status")
        if isinstance(status, str) and status:
            typer.echo(f"status: {status}")

        for field in SUMMARY_FIELDS:
            value = result.get(field)
            if value:
                typer.echo(f"{field}: {value}")

    def _format_progress_indicator(self, progress: int | None) -> str:
        if progress is None:
            frame = _SPINNER_FRAMES[self._spinner_index % len(_SPINNER_FRAMES)]
            self._spinner_index += 1
            return f"[{frame}]"

        clamped = max(0, min(100, progress))
        width = 24
        filled = round((clamped / 100) * width)
        bar = "#" * filled + "-" * (width - filled)
        return f"[{bar}] {clamped:>3d}%"

    def _flush_live_line(self) -> None:
        if not self._supports_live_updates:
            return
        if self._live_task_id is None:
            return
        typer.echo("")
        self._live_task_id = None
        self._live_line_len = 0



from __future__ import annotations

from pathlib import Path
from typing import Any

import httpx

from simplicity_cli.config import RuntimeConfig
from simplicity_cli.errors import CliError, ExitCode
from simplicity_cli.models import JSONDict


class SimplicityApiClient:
    def __init__(self, config: RuntimeConfig) -> None:
        self._timeout = config.request_timeout_seconds
        self._client = httpx.Client(
            base_url=config.base_url,
            timeout=self._timeout,
            headers={"X-API-Key": config.api_key},
            follow_redirects=True,
        )

    def __enter__(self) -> "SimplicityApiClient":
        return self

    def __exit__(self, exc_type: Any, exc: Any, tb: Any) -> None:
        self.close()

    def close(self) -> None:
        self._client.close()

    def upload_document_file(self, file_path: Path, document_type: str) -> JSONDict:
        with file_path.open("rb") as handle:
            files = {"file": (file_path.name, handle, "application/octet-stream")}
            return self._request(
                "POST",
                "/v1/documents/upload-document",
                params={"document_type": document_type},
                files=files,
            )

    def upload_document_url(self, url: str, document_type: str) -> JSONDict:
        return self._request(
            "POST",
            "/v1/documents/upload-document-url",
            params={"document_type": document_type},
            json={"url": url},
        )

    def create_autofill_form(
        self,
        form_file_document_id: str,
        source_document_ids: list[str] | None = None,
        context: str | None = None,
        instructions: str | None = None,
    ) -> JSONDict:
        body: JSONDict = {"form_file_document_id": form_file_document_id}
        if source_document_ids:
            body["source_document_ids"] = source_document_ids
        if context is not None:
            body["context"] = context
        if instructions is not None:
            body["instructions"] = instructions
        return self._request("POST", "/v1/forms/autofill", json=body)

    def autofill_existing_form(
        self,
        form_id: str,
        context: str | None = None,
        instructions: str | None = None,
    ) -> JSONDict:
        body: JSONDict = {}
        if context is not None:
            body["context"] = context
        if instructions is not None:
            body["instructions"] = instructions
        return self._request("POST", f"/v1/forms/{form_id}/autofill", json=body)

    def get_task(self, task_id: str) -> JSONDict:
        return self._request("GET", f"/v1/tasks/{task_id}")

    def get_form(self, form_id: str) -> JSONDict:
        return self._request("GET", f"/v1/forms/{form_id}")

    def get_document(self, document_id: str) -> JSONDict:
        return self._request("GET", f"/v1/documents/{document_id}")

    def download_file(self, download_url: str, destination: Path) -> None:
        destination.parent.mkdir(parents=True, exist_ok=True)
        try:
            with httpx.Client(timeout=self._timeout, follow_redirects=True) as download_client:
                with download_client.stream("GET", download_url) as response:
                    if response.status_code >= 400:
                        raise CliError(
                            f"file download failed with status {response.status_code}.",
                            exit_code=ExitCode.DOWNLOAD_ERROR,
                            code="download_http_error",
                            details={"status_code": response.status_code, "url": download_url},
                        )
                    with destination.open("wb") as handle:
                        for chunk in response.iter_bytes():
                            handle.write(chunk)
        except httpx.RequestError as exc:
            raise CliError(
                "file download failed due to network error.",
                exit_code=ExitCode.DOWNLOAD_ERROR,
                code="download_network_error",
                details={"url": download_url, "error": str(exc)},
            ) from exc

    def _request(self, method: str, path: str, **kwargs: Any) -> JSONDict:
        try:
            response = self._client.request(method, path, **kwargs)
        except httpx.RequestError as exc:
            raise CliError(
                "request to API failed.",
                exit_code=ExitCode.API_ERROR,
                code="request_error",
                details={"error": str(exc), "path": path, "method": method},
            ) from exc

        if response.status_code >= 400:
            self._raise_api_error(response)

        payload = self._safe_json(response)
        if isinstance(payload, dict):
            return payload
        return {"data": payload}

    def _raise_api_error(self, response: httpx.Response) -> None:
        payload = self._safe_json(response)
        if response.status_code in (401, 403):
            raise CliError(
                "authentication failed. Check your API key.",
                exit_code=ExitCode.AUTH_ERROR,
                code="auth_failed",
                details={"status_code": response.status_code, "response": payload},
            )

        error_code = "validation_error" if response.status_code == 422 else "api_error"
        message = self._extract_error_message(payload) or f"API request failed ({response.status_code})."
        raise CliError(
            message,
            exit_code=ExitCode.API_ERROR,
            code=error_code,
            details={"status_code": response.status_code, "response": payload},
        )

    @staticmethod
    def _safe_json(response: httpx.Response) -> Any:
        try:
            return response.json()
        except ValueError:
            return {"text": response.text}

    @staticmethod
    def _extract_error_message(payload: Any) -> str | None:
        if isinstance(payload, dict):
            detail = payload.get("detail")
            if isinstance(detail, str) and detail.strip():
                return detail.strip()
            if isinstance(detail, list) and detail:
                first = detail[0]
                if isinstance(first, dict):
                    msg = first.get("msg")
                    if isinstance(msg, str) and msg.strip():
                        return msg.strip()
        return None



from __future__ import annotations

import time
from pathlib import Path
from typing import Any

from simplicity_cli.client import SimplicityApiClient
from simplicity_cli.errors import CliError, ExitCode
from simplicity_cli.models import DownloadResolution, JSONDict
from simplicity_cli.output import OutputReporter


def load_optional_text(value: str | None, file_path: Path | None, label: str) -> str | None:
    if value is not None and file_path is not None:
        raise CliError(
            f"use either --{label} or --{label}-file, not both.",
            exit_code=ExitCode.USAGE_ERROR,
            code="invalid_input",
        )
    if file_path is None:
        return value
    try:
        return file_path.read_text(encoding="utf-8")
    except OSError as exc:
        raise CliError(
            f"failed to read {label} file: {file_path}",
            exit_code=ExitCode.USAGE_ERROR,
            code="file_read_error",
            details={"path": str(file_path), "error": str(exc)},
        ) from exc


def poll_task(
    client: SimplicityApiClient,
    task_id: str,
    poll_interval_seconds: float,
    max_wait_seconds: float,
    reporter: OutputReporter | None = None,
    task_label: str = "task",
) -> JSONDict:
    if poll_interval_seconds <= 0:
        raise CliError(
            "poll interval must be greater than 0 seconds.",
            exit_code=ExitCode.USAGE_ERROR,
            code="invalid_poll_interval",
        )
    if max_wait_seconds <= 0:
        raise CliError(
            "max wait must be greater than 0 seconds.",
            exit_code=ExitCode.USAGE_ERROR,
            code="invalid_max_wait",
        )

    start = time.monotonic()
    last_status: str | None = None
    last_progress_reported: int | None = None
    last_emit = 0.0
    not_found_reported = False
    while True:
        elapsed_seconds = time.monotonic() - start
        try:
            task = client.get_task(task_id)
        except CliError as exc:
            if _is_transient_task_not_found(exc):
                if reporter is not None and not not_found_reported:
                    reporter.task_update(
                        label=task_label,
                        task_id=task_id,
                        status="not-found-yet",
                        progress=None,
                        elapsed_seconds=elapsed_seconds,
                    )
                    not_found_reported = True
                if elapsed_seconds >= max_wait_seconds:
                    raise CliError(
                        f"task {task_id} did not become available within {max_wait_seconds:.0f} seconds.",
                        exit_code=ExitCode.TIMEOUT,
                        code="task_timeout",
                        details={"task_id": task_id},
                    ) from exc
                time.sleep(poll_interval_seconds)
                continue
            raise
        status = str(task.get("status", ""))
        progress = _extract_progress(task)
        if reporter is not None and _should_emit_task_update(
            status=status,
            progress=progress,
            last_status=last_status,
            last_progress_reported=last_progress_reported,
            elapsed_seconds=elapsed_seconds,
            last_emit=last_emit,
        ):
            reporter.task_update(
                label=task_label,
                task_id=task_id,
                status=status,
                progress=progress,
                elapsed_seconds=elapsed_seconds,
            )
            last_status = status
            last_progress_reported = progress
            last_emit = elapsed_seconds
        if status == "completed":
            return task
        if status == "failed":
            error_message = str(task.get("error_message") or "unknown task failure")
            raise CliError(
                f"task {task_id} failed: {error_message}",
                exit_code=ExitCode.TASK_FAILED,
                code="task_failed",
                details={"task_id": task_id, "task": task},
            )
        if elapsed_seconds >= max_wait_seconds:
            raise CliError(
                f"task {task_id} did not complete in {max_wait_seconds:.0f} seconds.",
                exit_code=ExitCode.TIMEOUT,
                code="task_timeout",
                details={"task_id": task_id},
            )
        time.sleep(poll_interval_seconds)


def run_fill_new(
    client: SimplicityApiClient,
    reporter: OutputReporter,
    *,
    form_file: Path | None,
    form_url: str | None,
    source_files: list[Path],
    source_urls: list[str],
    context: str | None,
    instructions: str | None,
    wait: bool,
    poll_interval_seconds: float,
    max_wait_seconds: float,
    download: bool,
    output_path: Path | None,
) -> JSONDict:
    reporter.phase("Upload form document")
    form_upload_task = (
        client.upload_document_file(form_file, document_type="FORM_DOCUMENT")
        if form_file is not None
        else client.upload_document_url(form_url or "", document_type="FORM_DOCUMENT")
    )
    form_upload_task_id = _required_task_id(form_upload_task, context_label="form upload")
    reporter.detail(f"task_id={form_upload_task_id}")

    reporter.phase("Wait for form upload")
    completed_form_upload = poll_task(
        client,
        form_upload_task_id,
        poll_interval_seconds=poll_interval_seconds,
        max_wait_seconds=max_wait_seconds,
        reporter=reporter,
        task_label="form upload",
    )
    form_document_id = _required_document_id(completed_form_upload, context_label="form upload")
    reporter.detail(f"form_document_id={form_document_id}")

    source_document_ids: list[str] = []
    source_total = len(source_files) + len(source_urls)
    source_index = 0
    for source_file in source_files:
        source_index += 1
        reporter.phase(f"Upload source document {source_index}/{source_total}")
        source_upload_task = client.upload_document_file(source_file, document_type="SOURCE_DOCUMENT")
        source_task_id = _required_task_id(source_upload_task, context_label="source upload")
        reporter.detail(f"task_id={source_task_id}")
        reporter.phase(f"Wait for source upload {source_index}/{source_total}")
        completed_source_upload = poll_task(
            client,
            source_task_id,
            poll_interval_seconds=poll_interval_seconds,
            max_wait_seconds=max_wait_seconds,
            reporter=reporter,
            task_label=f"source upload {source_index}/{source_total}",
        )
        source_document_id = _required_document_id(completed_source_upload, context_label="source upload")
        source_document_ids.append(source_document_id)
        reporter.detail(f"source_document_id={source_document_id}")

    for source_url in source_urls:
        source_index += 1
        reporter.phase(f"Upload source document {source_index}/{source_total}")
        source_upload_task = client.upload_document_url(source_url, document_type="SOURCE_DOCUMENT")
        source_task_id = _required_task_id(source_upload_task, context_label="source upload")
        reporter.detail(f"task_id={source_task_id}")
        reporter.phase(f"Wait for source upload {source_index}/{source_total}")
        completed_source_upload = poll_task(
            client,
            source_task_id,
            poll_interval_seconds=poll_interval_seconds,
            max_wait_seconds=max_wait_seconds,
            reporter=reporter,
            task_label=f"source upload {source_index}/{source_total}",
        )
        source_document_id = _required_document_id(completed_source_upload, context_label="source upload")
        source_document_ids.append(source_document_id)
        reporter.detail(f"source_document_id={source_document_id}")

    reporter.phase("Start autofill")
    autofill_task = client.create_autofill_form(
        form_file_document_id=form_document_id,
        source_document_ids=source_document_ids or None,
        context=context,
        instructions=instructions,
    )
    autofill_task_id = _required_task_id(autofill_task, context_label="autofill")
    reporter.detail(f"task_id={autofill_task_id}")

    if not wait:
        return _result_payload(
            status=str(autofill_task.get("status", "processing")),
            task_id=autofill_task_id,
            task=autofill_task,
        )

    reporter.phase("Wait for autofill")
    completed_autofill = poll_task(
        client,
        autofill_task_id,
        poll_interval_seconds=poll_interval_seconds,
        max_wait_seconds=max_wait_seconds,
        reporter=reporter,
        task_label="autofill",
    )
    form_id = _extract_form_id(completed_autofill)
    document_id = _extract_document_id(completed_autofill)
    if form_id:
        reporter.detail(f"form_id={form_id}")
    if document_id:
        reporter.detail(f"document_id={document_id}")
    payload = _result_payload(
        status=str(completed_autofill.get("status", "completed")),
        task_id=autofill_task_id,
        task=completed_autofill,
        form_id=form_id,
        document_id=document_id,
    )

    if not download:
        return payload

    reporter.phase("Download filled PDF")
    download_resolution = resolve_download_url(client, form_id=form_id, document_id=document_id)
    payload["download_url"] = download_resolution.url
    if not download_resolution.url:
        raise CliError(
            "autofill completed but no download URL was returned.",
            exit_code=ExitCode.DOWNLOAD_ERROR,
            code="missing_download_url",
            details=payload,
        )

    resolved_output_path = default_output_path(form_id=form_id, document_id=document_id, output_path=output_path)
    client.download_file(download_resolution.url, resolved_output_path)
    payload["download_path"] = str(resolved_output_path.resolve())
    reporter.detail(f"output={payload['download_path']}")
    return payload


def run_fill_existing(
    client: SimplicityApiClient,
    reporter: OutputReporter,
    *,
    form_id: str,
    context: str | None,
    instructions: str | None,
    wait: bool,
    poll_interval_seconds: float,
    max_wait_seconds: float,
    download: bool,
    output_path: Path | None,
) -> JSONDict:
    reporter.phase("Start autofill")
    autofill_task = client.autofill_existing_form(form_id=form_id, context=context, instructions=instructions)
    autofill_task_id = _required_task_id(autofill_task, context_label="autofill")
    reporter.detail(f"task_id={autofill_task_id}")

    if not wait:
        return _result_payload(
            status=str(autofill_task.get("status", "processing")),
            task_id=autofill_task_id,
            task=autofill_task,
            form_id=form_id,
        )

    reporter.phase("Wait for autofill")
    completed_autofill = poll_task(
        client,
        autofill_task_id,
        poll_interval_seconds=poll_interval_seconds,
        max_wait_seconds=max_wait_seconds,
        reporter=reporter,
        task_label="autofill",
    )
    document_id = _extract_document_id(completed_autofill)
    reporter.detail(f"form_id={form_id}")
    if document_id:
        reporter.detail(f"document_id={document_id}")
    payload = _result_payload(
        status=str(completed_autofill.get("status", "completed")),
        task_id=autofill_task_id,
        task=completed_autofill,
        form_id=form_id,
        document_id=document_id,
    )

    if not download:
        return payload

    reporter.phase("Download filled PDF")
    download_resolution = resolve_download_url(client, form_id=form_id, document_id=document_id)
    payload["download_url"] = download_resolution.url
    if not download_resolution.url:
        raise CliError(
            "autofill completed but no download URL was returned.",
            exit_code=ExitCode.DOWNLOAD_ERROR,
            code="missing_download_url",
            details=payload,
        )

    resolved_output_path = default_output_path(form_id=form_id, document_id=document_id, output_path=output_path)
    client.download_file(download_resolution.url, resolved_output_path)
    payload["download_path"] = str(resolved_output_path.resolve())
    reporter.detail(f"output={payload['download_path']}")
    return payload


def run_task_status(client: SimplicityApiClient, task_id: str) -> JSONDict:
    task = client.get_task(task_id)
    return _result_payload(
        status=str(task.get("status", "unknown")),
        task_id=task_id,
        task=task,
        form_id=_extract_form_id(task),
        document_id=_extract_document_id(task),
    )


def run_task_wait(
    client: SimplicityApiClient,
    reporter: OutputReporter,
    task_id: str,
    poll_interval_seconds: float,
    max_wait_seconds: float,
) -> JSONDict:
    reporter.phase("Wait for task completion")
    reporter.detail(f"task_id={task_id}")
    task = poll_task(
        client,
        task_id=task_id,
        poll_interval_seconds=poll_interval_seconds,
        max_wait_seconds=max_wait_seconds,
        reporter=reporter,
        task_label="task wait",
    )
    return _result_payload(
        status=str(task.get("status", "completed")),
        task_id=task_id,
        task=task,
        form_id=_extract_form_id(task),
        document_id=_extract_document_id(task),
    )


def resolve_download_url(
    client: SimplicityApiClient,
    *,
    form_id: str | None,
    document_id: str | None,
) -> DownloadResolution:
    form_payload: JSONDict | None = None
    if form_id:
        form_payload = client.get_form(form_id)
        form_url = form_payload.get("form_url")
        if isinstance(form_url, str) and form_url:
            return DownloadResolution(url=form_url, form_payload=form_payload)

    if document_id:
        document_payload = client.get_document(document_id)
        presigned_url = document_payload.get("presigned_url")
        if isinstance(presigned_url, str) and presigned_url:
            return DownloadResolution(
                url=presigned_url,
                form_payload=form_payload,
                document_payload=document_payload,
            )
        return DownloadResolution(url=None, form_payload=form_payload, document_payload=document_payload)

    return DownloadResolution(url=None, form_payload=form_payload)


def default_output_path(form_id: str | None, document_id: str | None, output_path: Path | None) -> Path:
    if output_path is not None:
        return output_path

    if form_id:
        token = form_id
    elif document_id:
        token = document_id
    else:
        raise CliError(
            "cannot determine output filename because both form_id and document_id are missing.",
            exit_code=ExitCode.DOWNLOAD_ERROR,
            code="missing_identifiers",
        )
    return Path.cwd() / f"{token}.filled.pdf"


def _required_task_id(payload: JSONDict, context_label: str) -> str:
    task_id = payload.get("task_id")
    if isinstance(task_id, str) and task_id:
        return task_id
    raise CliError(
        f"{context_label} response did not include task_id.",
        exit_code=ExitCode.API_ERROR,
        code="invalid_api_response",
        details={"response": payload},
    )


def _required_document_id(task: JSONDict, context_label: str) -> str:
    document_id = _extract_document_id(task)
    if document_id:
        return document_id
    raise CliError(
        f"{context_label} task completed without document_id.",
        exit_code=ExitCode.API_ERROR,
        code="missing_document_id",
        details={"task": task},
    )


def _extract_form_id(task: JSONDict) -> str | None:
    result_data = task.get("result_data")
    if isinstance(result_data, dict):
        for field in ("form_id", "formId"):
            value = result_data.get(field)
            if isinstance(value, str) and value:
                return value
    form_id = task.get("form_id")
    if isinstance(form_id, str) and form_id:
        return form_id
    return None


def _extract_document_id(task: JSONDict) -> str | None:
    result_data = task.get("result_data")
    if isinstance(result_data, dict):
        value = result_data.get("document_id")
        if isinstance(value, str) and value:
            return value
    document_id = task.get("document_id")
    if isinstance(document_id, str) and document_id:
        return document_id
    return None


def _result_payload(
    *,
    status: str,
    task_id: str | None = None,
    task: JSONDict | None = None,
    form_id: str | None = None,
    document_id: str | None = None,
    download_url: str | None = None,
    download_path: str | None = None,
) -> JSONDict:
    return {
        "status": status,
        "task_id": task_id,
        "task": task,
        "form_id": form_id,
        "document_id": document_id,
        "download_url": download_url,
        "download_path": download_path,
    }


def _is_transient_task_not_found(error: CliError) -> bool:
    details = error.details if isinstance(error.details, dict) else {}
    if details.get("status_code") != 404:
        return False

    response_payload = details.get("response")
    if isinstance(response_payload, dict):
        detail = response_payload.get("detail")
        if isinstance(detail, str) and detail.strip().lower() == "task not found":
            return True
    return False


def _extract_progress(task: JSONDict) -> int | None:
    value = task.get("progress")
    if isinstance(value, int):
        return value
    if isinstance(value, float):
        return int(value)
    return None


def _should_emit_task_update(
    *,
    status: str,
    progress: int | None,
    last_status: str | None,
    last_progress_reported: int | None,
    elapsed_seconds: float,
    last_emit: float,
) -> bool:
    if last_status is None:
        return True
    if status != last_status:
        return True
    if progress is not None and (
        last_progress_reported is None or progress >= last_progress_reported + 10 or progress == 100
    ):
        return True
    if elapsed_seconds - last_emit >= 15:
        return True
    return False



from __future__ import annotations

from dataclasses import dataclass
from typing import Any


JSONDict = dict[str, Any]


@dataclass(frozen=True)
class GlobalOptions:
    api_key: str | None
    json_output: bool
    request_timeout_seconds: float


@dataclass(frozen=True)
class DownloadResolution:
    url: str | None
    form_payload: JSONDict | None = None
    document_payload: JSONDict | None = None



from __future__ import annotations

import json
from pathlib import Path

import httpx
import pytest
import respx
from typer.testing import CliRunner

from simplicity_cli.cli import app

API_BASE = "https://api.simplicity.ai"
DEFAULT_ENV = {"SIMPLICITY_AI_API_KEY": "test-api-key"}


@pytest.fixture
def runner() -> CliRunner:
    return CliRunner()


@pytest.fixture(autouse=True)
def isolated_config_home(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> None:
    monkeypatch.setenv("XDG_CONFIG_HOME", str(tmp_path / "xdg"))


def invoke(runner: CliRunner, args: list[str], env: dict[str, str] | None = None):
    combined_env = dict(DEFAULT_ENV)
    if env:
        combined_env.update(env)
    return runner.invoke(app, args, env=combined_env)


def task_payload(task_id: str, status: str, **extra: object) -> dict[str, object]:
    payload: dict[str, object] = {
        "task_id": task_id,
        "status": status,
        "created_at": "2025-01-01T00:00:00Z",
    }
    payload.update(extra)
    return payload


def test_auth_missing_key_fails(runner: CliRunner, monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.delenv("SIMPLICITY_AI_API_KEY", raising=False)
    result = runner.invoke(app, ["status", "task-1"])
    assert result.exit_code == 2
    assert "SIMPLICITY_AI_API_KEY" in result.stdout


def test_no_args_prints_help(runner: CliRunner) -> None:
    result = runner.invoke(app, [])
    assert result.exit_code == 0
    assert "Usage:" in result.stdout


def test_login_flag_only_saves_key(runner: CliRunner, tmp_path: Path) -> None:
    xdg = tmp_path / "config-root"
    result = runner.invoke(app, ["--api-key", "saved-key-123"], env={"XDG_CONFIG_HOME": str(xdg)})
    assert result.exit_code == 0
    assert "API key saved to" in result.stdout

    config_path = xdg / "simplicity-cli" / "config.json"
    assert config_path.exists()
    payload = json.loads(config_path.read_text())
    assert payload["api_key"] == "saved-key-123"


def test_saved_key_is_used_without_env_or_flag(runner: CliRunner, tmp_path: Path) -> None:
    xdg = tmp_path / "config-root"
    login_result = runner.invoke(
        app,
        ["--api-key", "saved-key-xyz"],
        env={"XDG_CONFIG_HOME": str(xdg), "SIMPLICITY_AI_API_KEY": ""},
    )
    assert login_result.exit_code == 0

    with respx.mock(assert_all_called=True) as mock:
        def handler(request: httpx.Request) -> httpx.Response:
            assert request.headers["X-API-Key"] == "saved-key-xyz"
            return httpx.Response(200, json=task_payload("task-1", "processing"))

        mock.get(f"{API_BASE}/v1/tasks/task-1").mock(side_effect=handler)
        result = runner.invoke(
            app,
            ["status", "task-1"],
            env={"XDG_CONFIG_HOME": str(xdg), "SIMPLICITY_AI_API_KEY": ""},
        )
    assert result.exit_code == 0


def test_api_key_flag_overrides_env(runner: CliRunner) -> None:
    with respx.mock(assert_all_called=True) as mock:
        def handler(request: httpx.Request) -> httpx.Response:
            assert request.headers["X-API-Key"] == "flag-key"
            return httpx.Response(200, json=task_payload("task-1", "processing"))

        mock.get(f"{API_BASE}/v1/tasks/task-1").mock(side_effect=handler)
        result = invoke(
            runner,
            ["--api-key", "flag-key", "status", "task-1"],
            env={"SIMPLICITY_AI_API_KEY": "env-key"},
        )
    assert result.exit_code == 0


def test_top_level_help_uses_flat_commands(runner: CliRunner) -> None:
    result = invoke(runner, ["--help"])
    assert result.exit_code == 0
    assert "new" in result.stdout
    assert "existing" in result.stdout
    assert "status" in result.stdout
    assert "wait" in result.stdout
    assert "│ fill" not in result.stdout
    assert "│ task" not in result.stdout


def test_legacy_nested_alias_still_supported(runner: CliRunner) -> None:
    with respx.mock(assert_all_called=True) as mock:
        mock.get(f"{API_BASE}/v1/tasks/task-legacy").respond(200, json=task_payload("task-legacy", "processing"))
        result = invoke(runner, ["task", "status", "task-legacy"])
    assert result.exit_code == 0


def test_one_page_help_command(runner: CliRunner) -> None:
    result = invoke(runner, ["help"])
    assert result.exit_code == 0
    assert "one-page help" in result.stdout
    assert "simplicity-cli new" in result.stdout
    assert "simplicity-cli existing" in result.stdout
    assert "simplicity-cli status" in result.stdout
    assert "simplicity-cli wait" in result.stdout


def test_fill_new_rejects_invalid_form_input_combination(runner: CliRunner, tmp_path: Path) -> None:
    form_file = tmp_path / "form.pdf"
    form_file.write_bytes(b"form")
    result = invoke(
        runner,
        [
            "fill",
            "new",
            "--form-file",
            str(form_file),
            "--form-url",
            "https://example.com/form.pdf",
            "--context",
            "hello",
        ],
    )
    assert result.exit_code == 2
    assert "exactly one of --form-file or --form-url" in result.stdout


def test_fill_new_requires_source_or_context(runner: CliRunner, tmp_path: Path) -> None:
    form_file = tmp_path / "form.pdf"
    form_file.write_bytes(b"form")
    result = invoke(runner, ["new", "--form-file", str(form_file)])
    assert result.exit_code == 2
    assert "requires at least one source input" in result.stdout


def test_fill_new_local_files_downloads_default_output(
    runner: CliRunner,
    tmp_path: Path,
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    monkeypatch.chdir(tmp_path)
    form_file = tmp_path / "form.pdf"
    source_file = tmp_path / "source.pdf"
    form_file.write_bytes(b"form")
    source_file.write_bytes(b"source")

    with respx.mock(assert_all_called=True) as mock:
        mock.post(f"{API_BASE}/v1/documents/upload-document").mock(
            side_effect=[
                httpx.Response(200, json={"task_id": "upload-form-task"}),
                httpx.Response(200, json={"task_id": "upload-source-task"}),
            ]
        )
        mock.get(f"{API_BASE}/v1/tasks/upload-form-task").respond(
            200,
            json=task_payload("upload-form-task", "completed", document_id="form-doc"),
        )
        mock.get(f"{API_BASE}/v1/tasks/upload-source-task").respond(
            200,
            json=task_payload("upload-source-task", "completed", document_id="source-doc"),
        )
        mock.post(f"{API_BASE}/v1/forms/autofill").respond(
            200,
            json={"task_id": "autofill-task", "status": "processing"},
        )
        mock.get(f"{API_BASE}/v1/tasks/autofill-task").respond(
            200,
            json=task_payload("autofill-task", "completed", result_data={"form_id": "form-123"}),
        )
        mock.get(f"{API_BASE}/v1/forms/form-123").respond(
            200,
            json={
                "form_id": "form-123",
                "source_document_details": [],
                "form_url": "https://download.test/form-123.pdf",
            },
        )
        mock.get("https://download.test/form-123.pdf").respond(200, content=b"filled")
        result = invoke(
            runner,
            [
                "fill",
                "new",
                "--form-file",
                str(form_file),
                "--source-file",
                str(source_file),
            ],
        )

    assert result.exit_code == 0
    output_file = tmp_path / "form-123.filled.pdf"
    assert output_file.exists()
    assert output_file.read_bytes() == b"filled"


def test_fill_new_with_url_uploads(runner: CliRunner) -> None:
    with respx.mock(assert_all_called=True) as mock:
        mock.post(f"{API_BASE}/v1/documents/upload-document-url").mock(
            side_effect=[
                httpx.Response(200, json={"task_id": "upload-form-task"}),
                httpx.Response(200, json={"task_id": "upload-source-task"}),
            ]
        )
        mock.get(f"{API_BASE}/v1/tasks/upload-form-task").respond(
            200,
            json=task_payload("upload-form-task", "completed", document_id="form-doc"),
        )
        mock.get(f"{API_BASE}/v1/tasks/upload-source-task").respond(
            200,
            json=task_payload("upload-source-task", "completed", document_id="source-doc"),
        )
        mock.post(f"{API_BASE}/v1/forms/autofill").respond(
            200,
            json={"task_id": "autofill-task", "status": "processing"},
        )
        mock.get(f"{API_BASE}/v1/tasks/autofill-task").respond(
            200,
            json=task_payload("autofill-task", "completed", result_data={"form_id": "form-200"}),
        )
        result = invoke(
            runner,
            [
                "fill",
                "new",
                "--form-url",
                "https://example.com/form.pdf",
                "--source-url",
                "https://example.com/source.pdf",
                "--no-download",
            ],
        )
    assert result.exit_code == 0
    assert "task_id: autofill-task" in result.stdout


def test_fill_new_no_wait_does_not_poll_autofill(runner: CliRunner, tmp_path: Path) -> None:
    form_file = tmp_path / "form.pdf"
    form_file.write_bytes(b"form")
    with respx.mock(assert_all_called=False) as mock:
        mock.post(f"{API_BASE}/v1/documents/upload-document").respond(200, json={"task_id": "upload-form-task"})
        mock.get(f"{API_BASE}/v1/tasks/upload-form-task").respond(
            200,
            json=task_payload("upload-form-task", "completed", document_id="form-doc"),
        )
        mock.post(f"{API_BASE}/v1/forms/autofill").respond(
            200,
            json={"task_id": "autofill-task", "status": "processing"},
        )
        autofill_poll = mock.get(f"{API_BASE}/v1/tasks/autofill-task").respond(
            200,
            json=task_payload("autofill-task", "completed", result_data={"form_id": "form-201"}),
        )
        result = invoke(
            runner,
            [
                "fill",
                "new",
                "--form-file",
                str(form_file),
                "--context",
                "hello world",
                "--no-wait",
                "--no-download",
            ],
        )
    assert result.exit_code == 0
    assert not autofill_poll.called


def test_fill_existing_wait_downloads_by_form_id(
    runner: CliRunner,
    tmp_path: Path,
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    monkeypatch.chdir(tmp_path)
    with respx.mock(assert_all_called=True) as mock:
        mock.post(f"{API_BASE}/v1/forms/form-9/autofill").respond(
            200,
            json={"task_id": "autofill-task", "status": "processing"},
        )
        mock.get(f"{API_BASE}/v1/tasks/autofill-task").respond(
            200,
            json=task_payload("autofill-task", "completed"),
        )
        mock.get(f"{API_BASE}/v1/forms/form-9").respond(
            200,
            json={
                "form_id": "form-9",
                "source_document_details": [],
                "form_url": "https://download.test/form-9.pdf",
            },
        )
        mock.get("https://download.test/form-9.pdf").respond(200, content=b"filled-existing")
        result = invoke(runner, ["fill", "existing", "form-9", "--context", "Use existing info"])
    assert result.exit_code == 0
    assert (tmp_path / "form-9.filled.pdf").read_bytes() == b"filled-existing"


def test_task_wait_polls_until_completed(runner: CliRunner) -> None:
    with respx.mock(assert_all_called=True) as mock:
        poll_route = mock.get(f"{API_BASE}/v1/tasks/task-7").mock(
            side_effect=[
                httpx.Response(200, json=task_payload("task-7", "pending")),
                httpx.Response(200, json=task_payload("task-7", "processing")),
                httpx.Response(200, json=task_payload("task-7", "completed")),
            ]
        )
        result = invoke(
            runner,
            [
                "task",
                "wait",
                "task-7",
                "--poll-interval-seconds",
                "0.01",
                "--max-wait-seconds",
                "3",
            ],
        )
    assert result.exit_code == 0
    assert poll_route.call_count == 3


def test_task_wait_handles_failed_task(runner: CliRunner) -> None:
    with respx.mock(assert_all_called=True) as mock:
        mock.get(f"{API_BASE}/v1/tasks/task-failed").respond(
            200,
            json=task_payload("task-failed", "failed", error_message="autofill exploded"),
        )
        result = invoke(runner, ["task", "wait", "task-failed", "--poll-interval-seconds", "0.01"])
    assert result.exit_code == 5
    assert "autofill exploded" in result.stdout


def test_task_wait_timeout_returns_exit_code_6(runner: CliRunner) -> None:
    with respx.mock(assert_all_called=True) as mock:
        mock.get(f"{API_BASE}/v1/tasks/task-timeout").respond(200, json=task_payload("task-timeout", "processing"))
        result = invoke(
            runner,
            [
                "task",
                "wait",
                "task-timeout",
                "--poll-interval-seconds",
                "0.01",
                "--max-wait-seconds",
                "0.05",
            ],
        )
    assert result.exit_code == 6
    assert "did not complete" in result.stdout


def test_download_falls_back_to_document_presigned_url(
    runner: CliRunner,
    tmp_path: Path,
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    monkeypatch.chdir(tmp_path)
    form_file = tmp_path / "form.pdf"
    form_file.write_bytes(b"form")

    with respx.mock(assert_all_called=True) as mock:
        mock.post(f"{API_BASE}/v1/documents/upload-document").respond(200, json={"task_id": "upload-form-task"})
        mock.get(f"{API_BASE}/v1/tasks/upload-form-task").respond(
            200,
            json=task_payload("upload-form-task", "completed", document_id="form-doc"),
        )
        mock.post(f"{API_BASE}/v1/forms/autofill").respond(
            200,
            json={"task_id": "autofill-task", "status": "processing"},
        )
        mock.get(f"{API_BASE}/v1/tasks/autofill-task").respond(
            200,
            json=task_payload("autofill-task", "completed", result_data={"document_id": "filled-doc-1"}),
        )
        mock.get(f"{API_BASE}/v1/documents/filled-doc-1").respond(
            200,
            json={
                "document_id": "filled-doc-1",
                "file_key": "file-key",
                "presigned_url": "https://download.test/filled-doc-1.pdf",
            },
        )
        mock.get("https://download.test/filled-doc-1.pdf").respond(200, content=b"filled-doc")
        result = invoke(
            runner,
            [
                "fill",
                "new",
                "--form-file",
                str(form_file),
                "--context",
                "fallback to doc URL",
            ],
        )
    assert result.exit_code == 0
    assert (tmp_path / "filled-doc-1.filled.pdf").read_bytes() == b"filled-doc"


def test_json_output_is_single_object(runner: CliRunner) -> None:
    with respx.mock(assert_all_called=True) as mock:
        mock.get(f"{API_BASE}/v1/tasks/task-json").respond(200, json=task_payload("task-json", "processing"))
        result = invoke(runner, ["--json", "status", "task-json"])
    assert result.exit_code == 0
    lines = [line for line in result.stdout.splitlines() if line.strip()]
    assert len(lines) == 1
    payload = json.loads(lines[0])
    assert payload["command"] == "task status"
    assert payload["ok"] is True
    assert payload["error"] is None


@pytest.mark.parametrize("json_mode", [False, True])
def test_http_422_surfaces_validation_details(runner: CliRunner, json_mode: bool) -> None:
    with respx.mock(assert_all_called=True) as mock:
        mock.get(f"{API_BASE}/v1/tasks/task-bad").respond(
            422,
            json={"detail": [{"loc": ["query", "x"], "msg": "Invalid page", "type": "value_error"}]},
        )
        args = ["task", "status", "task-bad"]
        if json_mode:
            args.insert(0, "--json")
        result = invoke(runner, args)
    assert result.exit_code == 4
    if json_mode:
        payload = json.loads(result.stdout.strip())
        assert payload["error"]["code"] == "validation_error"
        assert payload["error"]["details"]["status_code"] == 422
    else:
        assert "Invalid page" in result.stdout
